#include "linesboard.h"
#include <QMessageBox>

linesBoard::linesBoard(QWidget *parent)
	: QLabel(parent)
{
	timer = new QTimer;
	connect (timer, SIGNAL(timeout()), this, SLOT(doStep()));
	randomTimer = new QTimer;
	connect (randomTimer, SIGNAL(timeout()), this, SLOT(setBallAfterPause()));
	randomDelTimer = new QTimer;
	connect (randomDelTimer, SIGNAL(timeout()), this, SLOT(delBallAfterPause()));
	boardIsCreated = 0;
	newGameClicked();

}


linesBoard::~linesBoard()
{
	delete backgroundPixmap;
}

void linesBoard::mousePressEvent (  QMouseEvent * mouseCondition )//
{
	if (mouseCondition->button() == Qt::LeftButton)
	{
		int x = mouseCondition->x();
		int y = geometry().height () - mouseCondition->y() ;
		int xCell = x / CELL_SIZE;
		int yCell = y / CELL_SIZE;
		continueGame(xCell, yCell);

	}
}

void  linesBoard::setBall (QColor ballColor,int xCell,int yCell)//
{
	int x = CELL_SIZE * xCell + CELL_SIZE/5;//
	int y = backgroundPixmap->height() - (CELL_SIZE * yCell) + CELL_SIZE/5 - CELL_SIZE;	
	
	QPainter painter;
	painter.begin(backgroundPixmap);

	QRadialGradient gradient(x,y, 50, x, y);//
	gradient.setColorAt(0, ballColor);
	gradient.setColorAt(1, Qt::white);
    
	QBrush brush(gradient);//
	painter.setBrush(brush);
	painter.setPen(QPen(Qt::black,1));//

	painter.drawEllipse(x, y, 30, 30);//
	setPixmap(*backgroundPixmap);

}

void linesBoard::deleteBall(int xCell, int yCell)//
{
	int x = CELL_SIZE * xCell;//
	int y = backgroundPixmap->height() - 1 - (CELL_SIZE * yCell) - CELL_SIZE;	
	
	QImage oneCell("one_cell.bmp");
	QPainter painter;
	painter.begin(backgroundPixmap);
	
	painter.drawImage(x, y, oneCell);

	setPixmap(*backgroundPixmap);

}


void linesBoard::resize (int newSize)
{
	QPixmap* buffer;
	buffer = new QPixmap(newSize * CELL_SIZE, newSize * CELL_SIZE);
	
	QPainter painter;
	painter.begin(buffer);


	if (newSize == 7)
		painter.drawPixmap(0, CELL_SIZE - 1, *backgroundPixmap);
	else
		if (newSize == 8)
			painter.drawPixmap(0, CELL_SIZE, *backgroundPixmap);
				else
					if (newSize == 9)
						painter.drawPixmap(0, CELL_SIZE, *backgroundPixmap);


	*backgroundPixmap = *buffer;
	setPixmap(*backgroundPixmap);
	
	emit newBoardSize(newSize * CELL_SIZE );
	
	for (int i = 0; i < newSize; i++ )
	{
		deleteBall(i,newSize - 1);
		deleteBall(newSize - 1, i);
	}
	painter.end();
	delete buffer;
}

void linesBoard::doStep() 
{
		if (path.size() != 0)
		{
			QString movingBallColorStr = path.dequeue ();
			QColor movingBallColor(movingBallColorStr);
			int xCellDel = (path.dequeue ()).toInt();
			int yCellDel = (path.dequeue ()).toInt();
			
			int xCellSet = (path.dequeue ()).toInt();
			int yCellSet = (path.dequeue ()).toInt();
			
			setBall(movingBallColor,xCellSet, yCellSet);
			deleteBall(xCellDel, yCellDel);
		}
		else
		{
			timer->stop();
		}
}

void linesBoard::setBallAfterPause()
{
	if (! timer->isActive() )
	{
		QColor curColor = QColor(queueForPauseSet.dequeue());
		int xCell = (queueForPauseSet.dequeue()).toInt();
		int yCell = (queueForPauseSet.dequeue()).toInt();
		setBall(curColor,xCell, yCell);
		if (queueForPauseSet.size() == 0)
		{
			randomTimer->stop();
		}
		
	}
	
}

void linesBoard::delBallAfterPause()
{
	if ( (! timer->isActive() ) && (! randomTimer->isActive() ) )
	{
		int xCell = (queueForPauseDel.dequeue()).toInt();
		int yCell = (queueForPauseDel.dequeue()).toInt();
		deleteBall(xCell, yCell);
		if (queueForPauseDel.size() == 0)
		{
			randomDelTimer->stop();
		}
		
	}
}





void linesBoard::newGameClicked()
{
	if (boardIsCreated == true) 
	{
		delete backgroundPixmap;
	}
	backgroundPixmap = new QPixmap;
	QImage image("background.bmp");
	*backgroundPixmap = QPixmap::fromImage(image);
	setPixmap(*backgroundPixmap);
	
	emit newBoardSize(BEGIN_SIZE * CELL_SIZE);
	continueGame(NEW_GAME_CODE, NEW_GAME_CODE);
}

void linesBoard::continueGame(int xCell, int yCell)
{
	QString commandString = mathLinesBoard.run(xCell, yCell);
	QStringList commandList = commandString.split(" ");
	
	QStringList::const_iterator i = commandList.begin();
	while (i != commandList.end())
	{
		if (*i == QString("SET") )
		{
			i++;

			QString colorName = *i;
			QColor cellColor;
			cellColor.setNamedColor(colorName);
			
		
			i++;
			xCell = (*i).toInt();

			i++;
			yCell = (*i).toInt();
			

				if (! timer->isActive() )
					setBall (cellColor, xCell, yCell);
				else
				{
					queueForPauseSet.enqueue(colorName);
					queueForPauseSet.enqueue(QString::number(xCell) );
					queueForPauseSet.enqueue(QString::number(yCell) );
					randomTimer->start(100);
				}
		}
		else 
			if (*i == QString("DEL") )
			{
				i++;
				xCell = (*i).toInt();

				i++;
				yCell = (*i).toInt();
				if ( (! timer->isActive() ) && (! randomTimer->isActive()) )
					deleteBall (xCell, yCell);
				else
				{
					queueForPauseDel.enqueue(QString::number(xCell) );
					queueForPauseDel.enqueue(QString::number(yCell) );
					randomDelTimer->start(100);
				}
			}
			else
				if (*i == "NEW_SIZE")
				{
					i++;
					int newSize = (*i).toInt (); 
					resize(newSize);
				}
				else
					if (*i == "SCORE_CHANGE")
					{
						i++;
						int newScore = (*i).toInt();
						emit scoreChanged (newScore);
					}
					else
					if (*i == "MOVE")
					{
						i++;
						path.enqueue(*i);
						i++;
						path.enqueue(*i);
						i++;
						path.enqueue(*i);
						i++;
						path.enqueue(*i);
						i++;
						path.enqueue(*i);
					}
					else
						if (*i == "START_MOVING")
						{
							timer->start(100);
						}
						else
							if (*i == "END_GAME")
							{
								i++;
								int finalScore = (*i).toInt();
								emit endGame(finalScore);

							}
		i++;
	}
}
