#include "mathboard.h"



MathBoard::MathBoard(QObject *parent)
	: QObject(parent)
{
	freeCells=0;
	curSize = BEGIN_SIZE;
	score = curSize * curSize;
}

MathBoard::~MathBoard()
{

}

MathBoard::MathBoard()
{
	curSize = BEGIN_SIZE;
	freeCells = curSize * curSize;
	score = 0;
	for (int i = 0; i < END_SIZE; i++)
	{
		for (int j = 0; j < END_SIZE; j++)
		{
			boardArr[i][j] = 0;
			

		}
	}

	prevX = -1, prevY = -1;


}

int MathBoard::move(int xCell, int yCell, int *xDelCell, int *yDelCell)
{
	for (int i = 0; i < END_SIZE; i++)
	{
		for (int j = 0; j < END_SIZE; j++)
		{
			copyArr[i][j] = boardArr[i][j];
		}
	}

	if (moveEnabled (xCell, yCell, prevX, prevY, 0) == true)
	{
		setMathBall(boardArr[prevX][prevY], xCell, yCell);
		setMathBall(0, prevX, prevY);
		*xDelCell = prevX;
		*yDelCell = prevY;
		prevX = -1;
		prevY = -1;
		
		return MOVE;
	}
	else
	{
		if ((xCell == -1) || (yCell == -1))
		{
			prevX = xCell;
			prevY = yCell;
		}
		else
			if (boardArr[xCell][yCell] != 0)
			{
				prevX = xCell;
				prevY = yCell;
			}
		return NO_MOVE;
	}
}

bool MathBoard::moveEnabled (int xCell, int yCell, int previousX, int previousY, int waveNumber)
{
	if ( (xCell == previousX) && (yCell == previousY) && (waveNumber !=0) && (0 == copyArr[previousX][previousY])  )
	{
		waveNumber++;
		copyArr[previousX][previousY] = 100+waveNumber;
		return true;
	}
	if ((previousX == -1) || (previousY == -1) )
		return false;
	if ( (xCell == previousX) && (yCell == previousY) && (waveNumber == 0) )
	{
		return false;
	}

	if ( (previousX >= curSize) || (previousY >= curSize) || (previousX < 0) || (previousY < 0) || (xCell >= curSize) || (yCell >= curSize) )
	{
		return false;
	}
	else
	{
		if ( ( 0 < copyArr[previousX][previousY] ) && (waveNumber != 0))// ,     ,    
		{
			return false;
		}
	}
	waveNumber++;
	copyArr[previousX][previousY] = 100+waveNumber;
	
	if (moveEnabled (xCell, yCell, previousX+1, previousY, waveNumber) )
	{
		return true;
	}
	else
	if (moveEnabled (xCell, yCell, previousX, previousY+1, waveNumber) )
	{
		return true;
	}
	else
	if (moveEnabled (xCell, yCell, previousX-1, previousY, waveNumber) )
	{
		return true;
	}
	else
	if (moveEnabled (xCell, yCell, previousX, previousY-1, waveNumber) )
	{
		return true;
	}
	else
		return false;
	return false;
}

QStack<int>  MathBoard::findPath(int beginX, int beginY, int endX, int endY)
{
	QStack <int> path;
	int xCell, yCell;
	
	int nearestX = beginX;
	int nearestY = beginY;
	do
	{
		xCell = nearestX,
		yCell = nearestY;
		if (isNextCoordsNearer(xCell, yCell, xCell, yCell + 1) ) 
		{
			nearestX = xCell;
			nearestY = yCell + 1;
		}
		else
		if (isNextCoordsNearer(xCell, yCell, xCell + 1, yCell) ) 
		{
			nearestX = xCell + 1;
			nearestY = yCell;
		}
		else
		if (isNextCoordsNearer(xCell, yCell, xCell, yCell - 1) ) 
		{
			nearestX = xCell;
			nearestY = yCell - 1;
		}
		else
		if (isNextCoordsNearer(xCell, yCell, xCell - 1, yCell) )
		{
			nearestX = xCell - 1;
			nearestY = yCell;
		}

		path.push(yCell);
		path.push(xCell);

	}
	while ( ! ((xCell == endX) && (yCell == endY)));
	
	
	return path;
}

bool MathBoard::isNextCoordsNearer(int curX, int curY, int nextX, int nextY)
{
	if ( (curX >= 0) && (curY >= 0) && (nextX >= 0) && (nextY >= 0) && (curX < curSize) && (curY < curSize) && (nextX < curSize) && (nextY < curSize) )
	{
		if ( (copyArr[nextX][nextY] >= 100) && (copyArr[nextX][nextY] < copyArr[curX][curY]) )
		{
			return true;
		}
	}
	return false;
}



void MathBoard::setMathBall(int color, int xCell, int yCell)
{
	if (color == 0)
	{
		freeCells++;
		freeCellsList.append(QPair<int, int> (xCell,yCell));
	}
	else
	{
		freeCellsList.removeAt( freeCellsList.indexOf(QPair<int, int> (xCell,yCell) ) );
		freeCells--;
		
	}
	boardArr[xCell][yCell] = color;
}

bool MathBoard::setRandomBall(int *xRandom, int *yRandom)
{
	if (freeCells > 0 )
	{
		int i = QTime::currentTime().msec() % freeCellsList.size();
		*xRandom = freeCellsList.at(i).first;
		*yRandom = freeCellsList.at(i).second;
		int rand;
		if (rand < 0)
			rand = -rand;
		setMathBall( ( (rand + QTime::currentTime().msec() ) % 5) + 1, *xRandom, *yRandom)  ;
		return true;
	}
	else
	{
		return false;
	}

}

QString MathBoard::color(int xCell, int yCell)
{
	switch (boardArr[xCell][yCell])
	{
	case 0:
		return QString("white");
	case 1:
		return QString("red");
	case 2:
		return QString("yellow");
	case 3:
		return QString("lime");
	case 4:
		return QString("blue");
	case 5:
		return QString("magenta");
	default:
		return QString("white");
	}

}

bool MathBoard::deleteNecessary()
{
	bool deleteDone = false;
	for (int i = 0; i < curSize; i++)
	{
		for (int j = 0; j < curSize; j++)
		{
			if (boardArr[i][j] != 0)
				if (deleteLineSearch(i,j))
					deleteDone = true;
		}
	}
	return deleteDone;
}

bool MathBoard::deleteLineSearch(int iFirstCell,int jFirstCell)
{
	int firstCell = boardArr[iFirstCell][jFirstCell];
	int i = iFirstCell;
	int j = jFirstCell;
	bool lineFinished = false;
	bool deleteMustBe = false;
	int lineLength = 0;
	for (int direction = 0; direction < 4; direction++)
	{
		lineLength=0;
		lineFinished = false;
		i = iFirstCell;
		j = jFirstCell;
		do
		{
			if ( ( i < 0) || (j < 0) || (i >= curSize) || (j >= curSize) )
			{
				lineFinished = true;
			}
			else
			{
				if ( (int)fabs((float) boardArr[i][j]) == (int) fabs( (float) firstCell) )
				{
					lineLength++;
					switch (direction)
					{
					case 0:
						i++;
						break;
					case 1:
						j++;
						break;
					case 2:
						i++;
						j++;
						break;
					case 3:
						i++;
						j--;
						break;
					}
				}
				else
					lineFinished = true;			
			}
		}
		while (! lineFinished);
		if (lineLength >= DELETE_LENGTH)
		{
			prepareLineDelete(iFirstCell, jFirstCell, i, j, direction);
			deleteMustBe =  true;
		}
	}
	return deleteMustBe;
}

void MathBoard::prepareLineDelete(int iFirstCell, int jFirstCell, int iLast, int jLast, int direction)
{
	int i=iFirstCell, j=jFirstCell;
	
	switch (direction)
	{
	case 0:
		while (i != iLast)
		{
			prepareCellDelete(i,j);
			if (iFirstCell > iLast)
			{
				i--;
			}
			if (iFirstCell < iLast)
			{
				i++;
			}
		}
		break;
	case 1:
		while (j != jLast)
		{
			prepareCellDelete(i,j);
			if (jFirstCell > jLast)
			{
				j--;
			}
			if (jFirstCell < jLast)
			{
				j++;
			}

		}
		break;

	case 2:
		while (i != iLast)
		{
			prepareCellDelete(i,j);
			if (iFirstCell > iLast)
			{
				i--;
				j--;
			}
			if (iFirstCell < iLast)
			{
				i++;
				j++;
			}

		}
		break;
	case 3:
		while (i != iLast)
		{
			prepareCellDelete(i,j);
			if (iFirstCell > iLast)
			{
				i--;
				j++;
			}
			if (iFirstCell < iLast)
			{
				i++;
				j--;
			}
		}
		break;
	}
}

void MathBoard::prepareCellDelete(int i, int j)
{
	if (boardArr[i][j] > 0)
	{
		boardArr[i][j] = 0 - boardArr[i][j];
	}
}

int MathBoard::getCurSize ()
{
	return curSize;
}

bool MathBoard::isCellForDelete(int i, int j)
{
	if (boardArr[i][j] < 0)
		return true;
	else
		return false;
}

int MathBoard::getScore()
{
	return score;
}

void MathBoard::addScore(int addPoints)
{
	score+=addPoints;
}

QString MathBoard::run(int xCell, int yCell)
{
	QString commandLine;
	if (xCell == NEW_GAME_CODE)
	{
		commandGameGraphicBegin(commandLine);
	}

	if (xCell != NEW_GAME_CODE)
	{
		int *xDelCell;
		xDelCell = new int;  
		int *yDelCell; 
		yDelCell = new int;
		
		*yDelCell = -1;
		*xDelCell = -1;
		if (move(xCell, yCell, xDelCell, yDelCell) == MOVE) 
		{
			QStack<int> path = findPath(xCell, yCell, *xDelCell, *yDelCell);
			QString curColor = color(xCell, yCell);
			commandGraphicMoving (path, curColor, commandLine);
	

			delete xDelCell;
			delete yDelCell;
				
			if ( deleteNecessary() ) 
			{
				int deleted = 0;
				for (int i = 0; i < getCurSize(); i++)
					for (int j = 0; j < getCurSize(); j++)
					{
						if ( isCellForDelete(i,j) ) 
						{
							setMathBall(0, i, j);
							commandDeleteGraphicBall (i, j , commandLine);
						
							deleted++;
						}
					}
				 
				addScore(deleted);
				commandSetGraphicScore(commandLine);
			}
			else
			{
				int *xNew = new int;
				int *yNew = new int;
				
				QString colorNew;
				for (int i=0; i<2; i++)
				{
					if (setRandomBall( xNew,  yNew))
					{
						commandSetGraphicBall (*xNew, *yNew, commandLine);	
					}
				}
				delete xNew;
				delete yNew;

				if ( deleteNecessary() )
				{
					int deleted = 0;
					for (int i = 0; i < getCurSize(); i++)
						for (int j = 0; j < getCurSize(); j++)
						{
							if ( isCellForDelete(i,j) ) 
							{
								setMathBall(0, i, j);
								commandDeleteGraphicBall (i, j, commandLine);

								deleted++;
							}
						}
					addScore(deleted);	
					commandSetGraphicScore(commandLine);
				}
				if ((freeCells == 0) && (curSize < END_SIZE))//  
				{
					curSize++;
					commandGraphicResize(commandLine);
					freeCells += 2*curSize - 1;
				}
				if ( ( (freeCells == 0) && (curSize == END_SIZE) ) || (freeCells == curSize*curSize) )// 
				{
						commandGameGraphicEnd(commandLine);
				}
			}
		}
	}
	return commandLine;
}

void MathBoard::commandDeleteGraphicBall (int xCellDelete, int yCellDelete, QString &commandString)
{
	commandString.append("DEL");
	commandString.append(" ");
	commandString.append(QString::number(xCellDelete) );
	commandString.append(" ");
	commandString.append(QString::number(yCellDelete) );
	commandString.append(" ");
}

void MathBoard::commandSetGraphicBall (int xCellAdd, int yCellAdd, QString &commandString)
{
	QString curColor = color(xCellAdd, yCellAdd);
	commandString.append("SET");
	commandString.append(" ");
	commandString.append(curColor);
	commandString.append(" ");
	commandString.append(QString::number(xCellAdd) );
	commandString.append(" ");
	commandString.append(QString::number(yCellAdd) );
	commandString.append(" ");
}

void MathBoard::commandSetGraphicScore(QString &commandString)
{
	commandString.append("SCORE_CHANGE");
	commandString.append(" ");
	commandString.append(QString::number(score));
	commandString.append(" ");

}

void MathBoard::commandGraphicMoving (QStack<int> path, QString curColor, QString &commandString)
{
	int previousX, previousY, nextX, nextY;
	previousX = path.pop();
	previousY = path.pop(); 
	while ( ! path.isEmpty() )
	{
		nextX = path.pop();
		nextY = path.pop();
		commandString.append("MOVE");
		commandString.append(" ");
		commandString.append(curColor);
		commandString.append(" ");
		commandString.append(QString::number(previousX));
		commandString.append(" ");
		commandString.append(QString::number(previousY));
		commandString.append(" ");
		commandString.append(QString::number(nextX));
		commandString.append(" ");
		commandString.append(QString::number(nextY));
		commandString.append(" ");

		previousX = nextX;
		previousY = nextY;

	}
	commandString.append("START_MOVING");
	commandString.append(" ");
}

void MathBoard::commandGraphicResize(QString &commandString)
{
	commandString.append("NEW_SIZE");
	commandString.append(" ");
	commandString.append(QString::number(curSize) );
	commandString.append(" ");

	for (int i = 0; i < curSize - 1; i++ )
	{
		freeCellsList.append(QPair<int, int> (i,curSize - 1));
		freeCellsList.append(QPair<int, int> (curSize - 1, i));
	}
	freeCellsList.append (QPair<int, int> (curSize - 1, curSize - 1) );
		
}

void MathBoard::commandGameGraphicEnd(QString &commandString)
{
	commandString.append("END_GAME");
	commandString.append(" ");
	commandString.append(QString::number(score) );
	commandString.append(" ");


}
	
void MathBoard::commandGameGraphicBegin(QString &commandString)
{
		commandSetGraphicScore(commandString);
		if (freeCells != 0)
			commandGameGraphicEnd(commandString);
		
		
		score = 0;

		
		freeCells = BEGIN_SIZE * BEGIN_SIZE;
		prevX = -1;
		prevY = -1;
		curSize = BEGIN_SIZE;
		freeCellsList.clear();
		for (int i = 0; i < curSize; i++)
			for (int j = 0; j< curSize; j++)
			{
				boardArr[i][j]=0;
				commandDeleteGraphicBall(i,j,commandString);
				freeCellsList.append(QPair<int,int>(i,j) );
			}
		int *xNew = new int; 
		int *yNew = new int;
		
		QString colorNew;
		for (int i=0; i<2; i++)
		{
			if (setRandomBall( xNew,  yNew))
			{
				commandSetGraphicBall (*xNew, *yNew, commandString);	
			}
		}
		delete xNew;
		delete yNew;
}
